#pragma once
#include "pyxie.h"

extern void PYXIE_EXPORT SaveImageToGallery(const char* value);
extern void PYXIE_EXPORT GetLocaleLanguage(char* locale, int localesize, char* language, int languagesize);
